package org.eclipse.birt.report.engine.emitter.xml;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.birt.report.engine.api.IRenderOption;
import org.eclipse.birt.report.engine.api.RenderOption;
import org.eclipse.birt.report.engine.content.IDataContent;
import org.eclipse.birt.report.engine.content.ILabelContent;
import org.eclipse.birt.report.engine.content.IReportContent;
import org.eclipse.birt.report.engine.content.IRowContent;
import org.eclipse.birt.report.engine.content.ITableContent;
import org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter;
import org.eclipse.birt.report.engine.emitter.IEmitterServices;
import org.eclipse.birt.report.engine.emitter.XMLWriter;
import org.eclipse.birt.report.engine.ir.Expression;
import org.eclipse.birt.report.engine.presentation.ContentEmitterVisitor;

public class XMLReportEmitter extends ContentEmitterAdapter {
  public final static String APPCONTEXT_XML_RENDER_CONTEXT = "XML_RENDER_CONTEXT"; //$NON-NLS-1$

  /**
   * the output format
   */
  public static final String OUTPUT_FORMAT_XML = "XML"; //$NON-NLS-1$

  /**
   * the default target report file name
   */
  public static final String REPORT_FILE = "report.xml"; //$NON-NLS-1$

  /**
   * output stream
   */
  protected OutputStream out = null;

  /**
   * the report content
   */
  protected IReportContent report;

  /**
   * the render options
   */
  protected IRenderOption renderOption;

  /**
   * should output the page header & footer
   */
  protected boolean outputMasterPageContent = true;

  /**
   * the <code>CSVWriter<code> object that is used to output CSV content
   */
  protected XMLWriter writer;

  /**
   * An Log object that <code>XMLReport</code> use to log the error, debug,
   * information messages.
   */
  protected static Logger logger = Logger.getLogger(XMLReportEmitter.class
      .getName());

  /**
   * emitter services
   */
  protected IEmitterServices services;

  /**
   * content visitor that is used to handle page header/footer
   */
  protected ContentEmitterVisitor contentVisitor;

  private Map<String, Expression> prop;

  /**
   * the constructor, instantiates the XML report emitter class as an
   * org.eclipse.birt.report.engine.presentation.ContentEmitterVisitor object
   * to perform emitter operations.
   */
  public XMLReportEmitter() {
    contentVisitor = new ContentEmitterVisitor(this);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.IContentEmitter#initialize(org
   * .eclipse.birt.report.engine.emitter.IEmitterServices)
   */
  /**
   * performs the following operations required to create an output stream
   * that writes the report contents to the XML file, Obtains a reference to
   * the IEmitterServices interface Instantiates the file and output stream
   * objects, using the specified settings Instantiates the XML file writer
   * object
   */
  @Override
  public void initialize(final IEmitterServices services) {
    System.out.println("BIRT PLUGIN initialize called");
    this.services = services;

    final Object fd = services.getOption(RenderOption.OUTPUT_FILE_NAME);
    File file = null;
    try {
      if (fd != null) {
        file = new File(fd.toString());
        final File parent = file.getParentFile();
        if (parent != null && !parent.exists()) {
          parent.mkdirs();
        }
        out = new BufferedOutputStream(new FileOutputStream(file
            + ".xml"));
      }
    } catch (final FileNotFoundException e) {
      logger.log(Level.WARNING, e.getMessage(), e);
    }

    if (out == null) {
      final Object value = services.getOption(RenderOption.OUTPUT_STREAM);
      if (value != null && value instanceof OutputStream) {
        out = (OutputStream) value;
      }
      else {
        try {
          file = new File(REPORT_FILE);
          out = new BufferedOutputStream(new FileOutputStream(file));
        } catch (final FileNotFoundException e) {

          logger.log(Level.SEVERE, e.getMessage(), e);
        }
      }
    }

    writer = new XMLWriter();
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter#start(org
   * .eclipse.birt.report.engine.content.IReportContent)
   */
  @Override
  public void start(final IReportContent report) {
    this.report = report;
    writer.open(out, "UTF-8");
    writer.startWriter();
    prop = report.getDesign().getUserProperties();
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter#end(org.
   * eclipse.birt.report.engine.content.IReportContent)
   */
  @Override
  public void end(final IReportContent report) {
    logger.log(Level.FINE, "[XMLReport] End report.");
    // writer.closeTag("ecoSpold");
    writer.endWriter();
    writer.close();
    if (out != null) {
      try {
        out.close();
      } catch (final IOException e) {
        logger.log(Level.WARNING, e.getMessage(), e);
      }
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter#startTable
   * (org.eclipse.birt.report.engine.content.ITableContent)
   */
  @Override
  public void startTable(final ITableContent table) {
    assert table != null;
    logger.log(Level.FINE, "[XMLReport] Start table");
    final boolean b = formatTableName(table.getName()).equals("datasetAttributs")
        || formatTableName(table.getName()).equals("subCategory")
        || formatTableName(table.getName()).equals("personTable")
        || formatTableName(table.getName()).equals("exchange");
    if (!b) {
      writer.openTag(formatTableName(table.getName()));
      if (table.getName().equals("ecospold") && prop != null) {
        Expression exp = prop.get("xmlns:xsi");
        writer.attribute("xmlns:xsi", exp.toString());
        exp = prop.get("xmlns");
        writer.attribute("xmlns", exp.toString());
        exp = prop.get("xsi:schemaLocation");
        writer.attribute("xsi:schemaLocation", exp);

      }
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter#endTable
   * (org.eclipse.birt.report.engine.content.ITableContent)
   */
  @Override
  public void endTable(final ITableContent table) {
    logger.log(Level.FINE, "[XMLReport] End table");

    // int size = valueList.size();

    final boolean b = formatTableName(table.getName()).equals("datasetAttributs")
        || formatTableName(table.getName()).equals("subCategory")
        || formatTableName(table.getName()).equals("personTable")
        || formatTableName(table.getName()).equals("exchange");

    if (!b) {
      // for (int i = size - 1; i > -1; i--) {
      // writer.attribute(formatName(nameList.get(i)), valueList.get(i));
      // }
      // nameList.clear();
      // valueList.clear();
      writer.closeTag(formatTableName(table.getName()));
    }
  }

  /*
   * Normalise les noms de table
   */
  private String formatTableName(String name) {
    final char c = '_';
    final int i = name.lastIndexOf(c);
    // while (i < name.length() && name.charAt(i) != c) {
    // i++;
    // }
    if (i > -1) {
      name = name.substring(i, name.length());
    }
    return formatName(name);
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter#startData
   * (org.eclipse.birt.report.engine.content.IDataContent)
   */
  @Override
  public void startData(final IDataContent data) {
    logger.log(Level.FINE, "[XMLReport] Start data");
    // Map<String, Object> map = data.getUserProperties();

    final String value = data.getText();
    // if (value == null)
    // value = "0";
    final String name = data.getName();
    final String bookmark = data.getBookmark();

    if (bookmark != null) {
      writer.text(value);
    }
    else {
      writer.attribute(formatTableName(name), value);
    }

  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter#startLabel
   * (org.eclipse.birt.report.engine.content.ILabelContent)
   */
  @Override
  public void startLabel(final ILabelContent label) {
    logger.log(Level.FINE, "[XMLReport] Start label");
    // boolean b = (boolean) label.getUserProperties().get("attribute");

    // if (b) {
    writer.attribute(formatTableName(label.getName()), label.getText());
    // }
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter#getOutputFormat
   * ()
   */
  @Override
  public String getOutputFormat() {
    return "xml";
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter#startRow
   * (org.eclipse.birt.report.engine.content.IRowContent)
   */
  @Override
  public void startRow(final IRowContent row) {
    logger.log(Level.FINE, "[XMLReport] Start row");
    final String rowBookmark = row.getBookmark();
    if (rowBookmark != null) {
      writer.openTag(rowFormat(rowBookmark));
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * org.eclipse.birt.report.engine.emitter.ContentEmitterAdapter#endRow(org
   * .eclipse.birt.report.engine.content.IRowContent)
   */
  @Override
  public void endRow(final IRowContent row) {
    logger.log(Level.FINE, "[XMLReport] End row");

    final Map<String, Object> map = row.getUserProperties();

    Double nbRow = null;
    Double index = null;
    int i = -1;
    if (map != null) {
      nbRow = (Double) map.get("nbRow");
      index = (Double) map.get("nbAllocations");
      i = (int) (index / 1.0);
    }

    final String rowBookmark = row.getBookmark();
    /*
     * Permet de traiter les allocations plus pr�cisement l'�l�ment
     * referenceToInputOutput
     */
    if (nbRow != null && rowBookmark.contains("allocation") && i > 0) {
      for (; i <= nbRow; i++) {
        writer.openTag("referenceToInputOutput");
        writer.text("" + i);
        writer.closeTag("referenceToInputOutput");
      }
    }

    if (rowBookmark != null) {
      writer.closeTag(rowFormat(rowBookmark));
    }
  }

  private String rowFormat(final String rowBookmark) {
    final char c = '_';
    final int i = rowBookmark.lastIndexOf(c);
    // while (i < rowBookmark.length() && rowBookmark.charAt(i) != c) {
    // i++;
    // }

    return rowBookmark.substring(0, i);
  }

  /*
   * M�thode qui normalise les noms Ex: "_text" devient "text"
   */
  private String formatName(final String text) {
    final char c = '_';
    int i = 0;
    while (i < text.length() && text.charAt(i) == c) {
      i++;
    }
    return text.substring(i);
  }

}
